//
//  FlagManager.swift
//  SampleApp
//
//  Created by Federico Zanetello on 5/14/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import AmitySDK

final class FlagManager {
    private let client: AmityClient
    private unowned let viewController: UIViewController
    private let userRepository: AmityUserRepository
    private let messageRepository: AmityMessageRepository

    init(client: AmityClient, viewController: UIViewController) {
        self.client = client
        self.viewController = viewController
        self.userRepository = AmityUserRepository(client: client)
        self.messageRepository = AmityMessageRepository(client: client)
    }

    func displayFlagAlertController(for message: AmityMessage) {
        guard let user: AmityUser = message.user else { return }

        Task { @MainActor in

            // retrieving user info
            var userFlagged: Bool = false

            do {
                let isFlagByMe = try await userRepository.isUserFlaggedByMe(withId: user.userId)
                userFlagged = isFlagByMe
            } catch {
                userFlagged = false
            }
            
            // retrieving message info
            var messageFlagged: Bool = false
            do {
                let isFlagByMe = try await messageRepository.isMessageFlaggedByMe(withId: message.messageId)
                messageFlagged = isFlagByMe
            } catch {
                messageFlagged = false
            }
            
            self.displayFlagAlertControlle(messageId: message.messageId,
                                            user: user,
                                            userIsFlagged: userFlagged,
                                            messageIsFlagged: messageFlagged)
        }
    }

    private func displayFlagAlertControlle(messageId: String,
                                           user: AmityUser,
                                           userIsFlagged: Bool,
                                           messageIsFlagged: Bool) {
        let alertController = UIAlertController(title: "Actions", message: nil, preferredStyle: .alert)
        let flagMessageAction = createFlagAction(messageId: messageId, flagged: messageIsFlagged)
        let flagUserAction = createFlagAction(userId: user.userId, flagged: userIsFlagged)

        let defaultAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)

        alertController.addAction(flagMessageAction)
        alertController.addAction(flagUserAction)
        alertController.addAction(defaultAction)

        viewController.present(alertController, animated: true, completion: nil)
    }

    /**
     creates a @p UIAlertAction for un/flagging the given messageId

     @param message the message to un/flag
     @return the @p UITableViewRowAction
     */
    func createFlagAction(messageId: String, flagged: Bool) -> UIAlertAction {
        let title: String = flagTitleForMessage(flagged: flagged)
        let flagMessage = UIAlertAction(title: title, style: .default, handler: { [weak self] _ in
            self?.flagAction(messageId: messageId, flagged: flagged)
        })
        return flagMessage
    }

    /**
     creates a @p UIAlertAction for un/flagging the given userId

     @param userId the userId to un/flag
     @return the @p UITableViewRowAction
     */
    private func createFlagAction(userId: String, flagged: Bool) -> UIAlertAction {
        let title: String = flagTitleForUser(flagged: flagged)

        let flagMessage = UIAlertAction(title: title, style: .default, handler: { [weak self] _ in
            self?.flagAction(userId: userId, flagged: flagged)
        })
        return flagMessage
    }

    /**
     returns the un/flag title for the given message

     @param flagged whether the message is flagged as of now
     @return the action title
     */
    func flagTitleForMessage(flagged: Bool) -> String {
        return flagged ? "Unflag Message" : "Flag Message"
    }

    /**
     returns the un/flag title for the given user

     @param flagged whether the message is flagged as of now
     @return the action title
     */
    private func flagTitleForUser(flagged: Bool) -> String {
        return flagged ? "Unflag User" : "Flag User"
    }

    /**
     Displays a simple UIAlertController with the given title, subtitle and an ok button, no actions are taken

     @param title the alert controller title
     @param subtitle the given subtitle
     */
    private func displayAlertController(title: String, subtitle: String?) {
        let alertController = UIAlertController.init(title: title, message: subtitle, preferredStyle: .alert)

        let defaultAction = UIAlertAction(title: "OK", style: .default, handler: nil)

        alertController.addAction(defaultAction)
        viewController.present(alertController, animated: true, completion: nil)
    }

    /**
     Handles the un/flag callback

     @param success the callback success parameter
     @param error the callback error parameter
     @param object a description of the object (e.g. "User" or "Message")
     @param isFlag whether this black is handling a flag callback or an unflag one
     */
    private func handleFlagBlock(success: Bool, error: Error?, object: String, isFlag: Bool) {
        if success {
            let title: String = String.init(format: "%@ %@flagged ✅", object, isFlag ? "" : "un")
            displayAlertController(title: title, subtitle: nil)
        } else {
            let title = "Error ❗️"
            displayAlertController(title: title, subtitle: error?.localizedDescription)
        }
    }

    /**
     called when the user wants to un/flag the message for given givenId:
     this method will execute the action and display a @p UIAlertController with the outcome

     @param messageId of the message to un/flag
     */
    private func flagAction(messageId: String, flagged: Bool) {
        Task { @MainActor in
            if flagged {
                do {
                    let success = try await messageRepository.unflagMessage(withId: messageId)
                    self.handleFlagBlock(success: success, error: nil, object: "message", isFlag: false)
                } catch {
                    self.handleFlagBlock(success: false, error: error, object: "message", isFlag: false)
                }
                Log.add(info: messageRepository)
            } else {
                do {
                    let success = try await messageRepository.flagMessage(withId: messageId)
                    self.handleFlagBlock(success: success, error: nil, object: "message", isFlag: true)
                } catch {
                    self.handleFlagBlock(success: false, error: error, object: "message", isFlag: true)
                }
                Log.add(info: messageRepository)
            }
        }
    }

    /**
     called when the user wants to un/flag the given user:
     this method will execute the action and display a @p UIAlertController with the outcome

     @param userId the user to un/flag
     */
    private func flagAction(userId: String, flagged: Bool) {
        if flagged {
            Task { @MainActor in
                do {
                    let result = try await userRepository.unflagUser(withId: userId)
                    self.handleFlagBlock(success: result, error: nil, object: "User", isFlag: false)
                } catch {
                    self.handleFlagBlock(success: false, error: error, object: "User", isFlag: false)
                }
            }
        } else {
            Task { @MainActor in
                do {
                    let result = try await userRepository.flagUser(withId: userId)
                    self.handleFlagBlock(success: result, error: nil, object: "User", isFlag: true)
                } catch {
                    self.handleFlagBlock(success: false, error: error, object: "User", isFlag: true)
                }
            }
        }
    }
    
    
    
}
