//
//  ViewController.swift
//  SampleAppBroadcaster
//
//  Created by Nutchaphon Rewik on 17/2/2564 BE.
//

import UIKit
import AmityLiveVideoBroadcastKit
import AVFoundation

class BroadcastViewController: UIViewController {

    private var isBroadcasting = false
    
    private var broadcaster: AmityStreamBroadcaster?
    
    private var hasSetup = false
    
    private var canvasBounds = CGRect.zero
    
    @IBOutlet private weak var canvas: UIView!
    @IBOutlet private weak var startButton: UIButton!
    @IBOutlet private weak var stopButton: UIButton!
    @IBOutlet private weak var videoResolutionSegmentedControl: UISegmentedControl!
    
    @IBOutlet private weak var stateView: UIView!
    @IBOutlet private weak var stateLabel: UILabel!
    
    @IBOutlet private weak var settingsView: UIView!
    @IBOutlet private weak var titleTextField: UITextField!
    @IBOutlet private weak var descriptionTextField: UITextField!
    
    @IBOutlet private weak var permissionView: UIView!
    @IBOutlet private weak var authorizeCameraButton: UIButton!
    @IBOutlet private weak var authorizeMicrophoneButton: UIButton!
    
    deinit {
        broadcaster?.stopPublish()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        stateView.clipsToBounds = true
        stateView.layer.cornerRadius = 8.0
        titleTextField.delegate = self
        descriptionTextField.delegate = self
        updateUI()
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        canvasBounds = canvas.bounds
        updateUI()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        // If the permission is authorized, we can try setup broadcaster now.
        let auth = AVCaptureDevice.authorizationStatus(for: .video)
        switch auth {
        case .authorized:
            trySetupBroadcaster()
        default:
            break
        }
        
    }
    
    override var shouldAutorotate: Bool {
        return true
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return [.portrait]
    }
    
    private func trySetupBroadcaster() {
        if !hasSetup && permissionsGranted() {
            hasSetup = true
            setupBroadcaster()
        }
    }
    
    private func setupBroadcaster() {
        
        broadcaster = AmityStreamBroadcaster(client: AppManager.shared.client!)
        broadcaster?.delegate = self
        broadcaster?.videoResolution = canvas.bounds.size
        broadcaster!.setup(with: getConfig())
        
        // Embed broadcaster.previewView
        broadcaster!.previewView.translatesAutoresizingMaskIntoConstraints = false
        canvas.addSubview(broadcaster!.previewView)
        
        NSLayoutConstraint.activate([
            broadcaster!.previewView.centerYAnchor.constraint(equalTo: canvas.centerYAnchor),
            broadcaster!.previewView.centerXAnchor.constraint(equalTo: canvas.centerXAnchor),
            broadcaster!.previewView.widthAnchor.constraint(equalToConstant: canvasBounds.size.width),
            broadcaster!.previewView.heightAnchor.constraint(equalToConstant: canvasBounds.size.height)
        ])
        
    }
    
    private func getConfig() -> AmityStreamBroadcasterConfiguration {
        let config = AmityStreamBroadcasterConfiguration()
        config.canvasFitting = .fill
        config.bitrate = 3_000_000 // 3mbps
        config.frameRate = .fps30
        return config
    }
    
    private func updateUI() {
        
        let _permissionsGranted = permissionsGranted()
        canvas.isHidden = !_permissionsGranted
        permissionView.isHidden = _permissionsGranted
        startButton.isHidden = isBroadcasting
        stopButton.isHidden = !isBroadcasting
        settingsView.isHidden = isBroadcasting
        stateView.isHidden = !isBroadcasting
        
        updateStateView()
        
        if AVCaptureDevice.authorizationStatus(for: .video) != .authorized {
            authorizeCameraButton.setTitle("1. Authorize Camera Access", for: .normal)
            authorizeCameraButton.isEnabled = true
        } else {
            authorizeCameraButton.setTitle("1. Authorize Camera Access ✅", for: .normal)
            authorizeCameraButton.isEnabled = false
        }
        
        if AVAudioSession.sharedInstance().recordPermission != .granted {
            authorizeMicrophoneButton.setTitle("2. Authorize Microphone Access", for: .normal)
            authorizeMicrophoneButton.isEnabled = true
        } else {
            authorizeMicrophoneButton.setTitle("2. Authorize Microphone Access ✅", for: .normal)
            authorizeMicrophoneButton.isEnabled = false
        }
        
    }
    
    private func updateStateView() {
        guard let broadcaster = broadcaster else {
            return
        }
        switch broadcaster.state {
        case .idle:
            stateLabel.text = "Idle"
        case .connecting:
            stateLabel.text = "Connecting"
        case .connected:
            stateLabel.text = "Live"
        case .disconnected:
            stateLabel.text = "Disconnected"
        @unknown default:
            assertionFailure("Unhandled case")
        }
    }
    
    private func permissionsGranted() -> Bool {
        let auth = AVCaptureDevice.authorizationStatus(for: .video)
        if auth != .authorized {
            return false
        }
        if AVAudioSession.sharedInstance().recordPermission != .granted {
            return false
        }
        return true
    }
    
    private func presentStopLiveConfirmationDialogue() {
        
        let alertController = UIAlertController(title: "Do want to stop the current live video session?", message: nil, preferredStyle: .alert)
        
        let stop = UIAlertAction(title: "Stop", style: .destructive) { [weak self] action in
            guard let strongSelf = self else { return }
            strongSelf.reallyStopLiveStream()
        }
        
        let cancel = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        
        alertController.addAction(stop)
        alertController.addAction(cancel)
        
        present(alertController, animated: true, completion: nil)
    }
    
    private func reallyStopLiveStream() {
        broadcaster?.stopPublish()
        isBroadcasting = false
        titleTextField.text = nil
        descriptionTextField.text = nil
        updateUI()
    }
    
    @IBAction func videoResolutionSegmentedControlValueChanged() {
        
        let sizes = [
            canvasBounds.size,
            CGSize(width: 1920, height: 1080),
            CGSize(width: 1280, height: 720),
            CGSize(width: 852, height: 480)
        ]
        
        let sizeToChoose = sizes[videoResolutionSegmentedControl.selectedSegmentIndex]
        
        broadcaster?.videoResolution = sizeToChoose
        
    }
    
    @IBAction func startButtonDidTouch() {
        
        let _title = titleTextField.text ?? ""
        let _description = descriptionTextField.text ?? ""
        
        let title = _title.isEmpty ? UUID().uuidString : _title
        let description = _description.isEmpty ? UUID().uuidString : _description
        
        broadcaster?.startPublishNewStream(title: title, description: description, thumbnailImage: nil)
        
        isBroadcasting = true
        updateUI()
        
    }
    
    @IBAction func stopButtonDidTouch() {
        presentStopLiveConfirmationDialogue()
    }
    
    @IBAction func switchCameraButtonDidTouch() {
        
        guard let _broadcaster = broadcaster else {
            return
        }
        
        // Toggle camera position.
        switch _broadcaster.cameraPosition {
        case .front:
            broadcaster?.cameraPosition = .back
        case .back:
            broadcaster?.cameraPosition = .front
        @unknown default:
            assertionFailure("Unhandled case")
        }
        
    }
    
    @IBAction func authorizeButtonDidTouch(_ sender: Any) {
        
        let auth = AVCaptureDevice.authorizationStatus(for: .video)
        switch auth {
        case.notDetermined:
            // The user has not yet accept or deny the permission, so we can ask directly.
            // NOTE: Please make sure you've added `NSCameraUsageDescription` into info.plist.
            AVCaptureDevice.requestAccess(for: .video) { [weak self] granted in
                DispatchQueue.main.async {
                    self?.updateUI()
                    // After the user authorize the permission to camera and microphone.
                    // We can try setup broadcaster.
                    self?.trySetupBroadcaster()
                }
            }
        case .denied, .restricted:
            // The user has already denied the permission, so the only way is to let user do this manually.
            // By going into the setting page of the app.
            if
                let bundleIdentifier = Bundle.main.bundleIdentifier,
                let appSettings = URL(string: UIApplication.openSettingsURLString + bundleIdentifier) {
                
                UIApplication.shared.open(appSettings)
                
            }
        case .authorized:
            break
        @unknown default:
            assertionFailure("Unhandled case")
        }
        
    }
    
    @IBAction func authorizeMicrophoneButtonDidTouch() {
        
        switch AVAudioSession.sharedInstance().recordPermission {
        case .undetermined:
            // The user has not yet accept or deny the permission, so we can ask directly.
            // NOTE: Please make sure you've added `NSMicrophoneUsageDescription`.
            AVAudioSession.sharedInstance().requestRecordPermission { [weak self] granted in
                DispatchQueue.main.async {
                    self?.updateUI()
                    // After the user authorize the permission to camera and microphone.
                    // We can try setup broadcaster.
                    self?.trySetupBroadcaster()
                }
            }
        case .denied:
            // The user has already denied the permission, so the only way is to let user do this manually.
            // By going into the setting page of the app.
            if
                let bundleIdentifier = Bundle.main.bundleIdentifier,
                let appSettings = URL(string: UIApplication.openSettingsURLString + bundleIdentifier) {
                
                UIApplication.shared.open(appSettings)
                
            }
        case .granted:
            break
        @unknown default:
            fatalError("Unhandled case")
        }
        
    }
    

}

extension BroadcastViewController: UITextFieldDelegate {
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true
    }
    
}

extension BroadcastViewController: AmityStreamBroadcasterDelegate {
    
    func amityStreamBroadcasterDidUpdateState(_ broadcaster: AmityStreamBroadcaster) {
        updateStateView()
    }
    
}
